@@ -1,40 +1,48 @@
+"""
+Utility functions
+"""
 import logging
+from typing import List, Tuple, Dict
 
-import matplotlib.patches as mpatches
+import matplotlib.patches as m_patches
 import matplotlib.transforms as trn
 import numpy as np
 from matplotlib.path import Path as mPath
 
 _logger = logging.getLogger(__name__)
 
-RATIO_OPTIONS = {"golden_ratio", "equal", "from_rows"}
+RATIO_OPTIONS = {'golden_ratio', 'equal', 'from_rows'}
 
 
 def add_values_to_bars(
-    axis,
-    type="bar",
-    position="c",
-    format="{:.0f}",
-    x_offset=0,
-    y_offset=0,
-    color="k",
-    horizontalalignment="center",
-    verticalalignment="center",
+        axis,
+        bar_type='bar',
+        position='c',
+        label_format='{:.0f}',
+        x_offset=0,
+        y_offset=0,
+        color='k',
+        horizontalalignment='center',
+        verticalalignment='center',
 ):
     """
     Add the values of the bars as number in the center
 
+    This function adds the numerical value of each bar to the bar itself. By default, it will be
+    centered on the bar, but you can change this by using the `position` parameter. The value is
+    formatted according to the `format` parameter.
+
     Parameters
     ----------
-    axis : `mpl.pyplot.axes.Axes` object
+    axis: `mpl.pyplot.axes.Axes` object
         Axis containing the bar plot
+    bar_type: {"bar", "barh"}
+        Direction of the bars. Default = "bar", meaning vertical bars. Alternatively, you need to
+        specify "barh" for horizontal bars.
     position: {"c", "t", "l", "r", "b"}, optional
         Location of the numbers, where "c" is center, "t" is top, "l" is left, "r" is right and "b"
         is bottom. Default = "c"
-    type: {"bar", "barh"}
-        Direction of the bars. Default = "bar", meaning vertical bars. Alternatively you need to
-        specify "barh" for horizontal bars.
-    format: str, optional
+    label_format: str, optional
         Formatter to use for the numbers. Default = "{:.0f}" (remove digits from float)
     x_offset: float, optional
         x offset in pt. Default = 0
@@ -46,42 +54,53 @@ def add_values_to_bars(
         Horizontal alignment of the numbers. Default = "center"
     verticalalignment: str, optional
         Vertical alignment of the numbers Default = "center"
-    ):
     """
 
     # voeg percentage to aan bars
     for patch in axis.patches:
         b = patch.get_bbox()
+        # calculate the center of the bar
         cx = (b.x1 + b.x0) / 2
         cy = (b.y1 + b.y0) / 2
+
+        # calculate the height and width of the bar
         hh = b.y1 - b.y0
         ww = b.x1 - b.x0
-        if position == "c":
+
+        # determine the position of the text
+        if position == 'c':
+            # center
             (px, py) = (cx, cy)
-        elif position == "t":
+        elif position == 't':
+            # top
             (px, py) = (cx, cy + hh / 2)
-        elif position == "b":
+        elif position == 'b':
+            # bottom
             (px, py) = (cx, cy - hh / 2)
-        elif position == "l":
+        elif position == 'l':
+            # left
             (px, py) = (cx - ww / 2, cy)
-        elif position == "r":
+        elif position == 'r':
+            # right
             (px, py) = (cx + ww / 2, cy)
         else:
             raise ValueError(f"position = {position} not recognised. Please check")
 
-        # add offsets
+        # add the offsets
         (px, py) = (px + x_offset, py + y_offset)
 
-        if type == "bar":
+        # determine the value of the bar
+        if bar_type == 'bar':
             value = hh
-        elif type == "barh":
+        elif bar_type == 'barh':
             value = ww
         else:
-            raise ValueError(f"type = {type} not recognised. Please check")
+            raise ValueError(f"type = {bar_type} not recognised. Please check")
 
         # make the value string using the format specifier
-        value_string = format.format(value)
+        value_string = label_format.format(value)
 
+        # add the value to the plot
         axis.annotate(
             value_string,
             (px, py),
@@ -92,20 +111,54 @@ def add_values_to_bars(
 
 
 def add_cbs_logo_to_plot(
-    fig,
-    axes=None,
-    margin_x_in_mm=6.0,
-    margin_y_in_mm=6.0,
-    x0=0,
-    y0=0,
-    width=None,
-    height=None,
-    zorder_start=1,
-    fillcolor="cbs:highchartslichtgrijs",
-    edgecolor="cbs:logogrijs",
+        fig,
+        axes=None,
+        margin_x_in_mm=6.0,
+        margin_y_in_mm=6.0,
+        x0=0,
+        y0=0,
+        width=None,
+        height=None,
+        zorder_start=1,
+        fillcolor='cbs:highchartslichtgrijs',
+        edgecolor='cbs:logogrijs',
 ):
-    # maak een box met de coordinaten van de linker onderhoek van het grijze vierkant in axis
-    # fractie coordinaten
+    """
+    Add the CBS logo to the plot.
+
+    Parameters
+    ----------
+    fig : `matplotlib.figure.Figure` object
+        The total canvas of the Figure
+    axes : `matplotlib.axes.Axes` object, optional
+        The axes of the plot to add a box
+    margin_x_in_mm : float, optional
+        The margin between the left of the Figure and the logo in mm. Default = 6.0
+    margin_y_in_mm : float, optional
+        The margin between the bottom of the Figure and the logo in mm. Default = 6.0
+    x0 : float, optional
+        The x-coordinate of the bottom left corner of the gray square in axis fraction coordinates.
+        Default = 0
+    y0 : float, optional
+        The y-coordinate of the bottom left corner of the gray square in axis fraction coordinates.
+        Default = 0
+    width : float, optional
+        The width of the gray square in axis fraction coordinates. Default = 1
+    height : float, optional
+        The height of the gray square in axis fraction coordinates. Default = 1
+    zorder_start : int, optional
+        The zorder of the first path of the logo. The zorders of the other paths are incremented by 1.
+        Default = 1
+    fillcolor : str, optional
+        The fill color of the logo. Default = "cbs:highchartslichtgrijs"
+    edgecolor : str, optional
+        The edge color of the logo. Default = "cbs:logogrijs"
+
+    Returns
+    -------
+    None
+
+    """
     if width is None:
         ww = 1
     else:
@@ -118,14 +171,14 @@ def add_cbs_logo_to_plot(
     if axes is not None:
         tb = trn.Bbox.from_bounds(x0, y0, ww, hh).transformed(axes.transAxes)
 
-        # bereken de linker onderhoek van het figure in Figure coordinaten (pt van linker onderhoek)
+        # Calculate the bottom left corner of the figure in Figure coordinates (pt of the bottom left corner)
         x0 = tb.x0 + (margin_x_in_mm / 25.4) * fig.dpi
         y0 = tb.y0 + (margin_y_in_mm / 25.4) * fig.dpi
     else:
         x0 = (margin_x_in_mm / 25.4) * fig.dpi
         y0 = (margin_y_in_mm / 25.4) * fig.dpi
 
-    all_points = get_cbs_logo_points()
+    all_points = _get_cbs_logo_points()
 
     if axes is not None:
         trans = axes.transAxes
@@ -146,7 +199,7 @@ def add_cbs_logo_to_plot(
                 color = edgecolor
             else:
                 color = fillcolor
-            poly = mpatches.PathPatch(
+            poly = m_patches.PathPatch(
                 tr_path, fc=color, linewidth=0, zorder=zorder, transform=trans
             )
             poly.set_clip_on(False)
@@ -157,26 +210,28 @@ def add_cbs_logo_to_plot(
             zorder += 1
 
 
-def get_cbs_logo_points(logo_width_in_mm=3.234, logo_height_in_mm=4.995, rrcor=0.171):
+def _get_cbs_logo_points(logo_width_in_mm=3.234, rrcor=0.171):
     """
-    Maak een array met de letters van het CBS logog
+    Generate the CBS logo points as a list of numpy arrays.
+
+    This function returns the points required to draw the CBS logo,
+    consisting of the letters 'C', 'B', and 'S'. Each letter is represented
+    by a set of points describing its outline and inner details, using
+    matplotlib path codes for path drawing.
 
     Parameters
     ----------
-    logo_width_in_mm: float
-        Breeedte van het logo
-    logo_height_in_mm: float
-        Hoogte van het logo
-    rrcor: float
-        Radius of corners
+    logo_width_in_mm : float, optional
+        The width of the logo in millimeters. Default is 3.234.
+    rrcor : float, optional
+        The radius of the rounded corners in millimeters. Default is 0.171.
 
     Returns
     -------
     list
-        List met 3 Nx2 arrays
-
+        A list containing numpy arrays. Each numpy array represents a part
+        of a letter and contains point coordinates and path codes for the logo.
     """
-
     ww = logo_width_in_mm
 
     # punten C, beginnen links onder, tegen klok in, binnen en buiten kant
@@ -275,8 +330,8 @@ def get_cbs_logo_points(logo_width_in_mm=3.234, logo_height_in_mm=4.995, rrcor=0
         ),
     ]
 
-    # de punten van de S, beginnende linksboven, tegen de klok in. Eerst array is de
-    # buitenkant, tweede array is de binnenkant
+    # De punten van de S, beginnende linksboven, tegen de klok in.
+    # Eerst array is de buitenkant, tweede array is de binnenkant
     points_s = [
         np.array(
             list(
@@ -326,54 +381,70 @@ def get_cbs_logo_points(logo_width_in_mm=3.234, logo_height_in_mm=4.995, rrcor=0
 
 
 def add_axis_label_background(
-    fig,
-    axes,
-    alpha=1,
-    margin=0.05,
-    x0=None,
-    y0=None,
-    loc="east",
-    radius_corner_in_mm=1,
-    logo_margin_x_in_mm=1,
-    logo_margin_y_in_mm=1,
-    add_logo=True,
-    aspect=None,
-    backgroundcolor="cbs:highchartslichtgrijs",
-    logo_fillcolor="cbs:highchartslichtgrijs",
-    logo_edgecolor="cbs:logogrijs",
+        fig,
+        axes,
+        alpha=1,
+        margin=0.05,
+        x0=None,
+        y0=None,
+        loc='east',
+        radius_corner_in_mm=1,
+        logo_margin_x_in_mm=1,
+        logo_margin_y_in_mm=1,
+        add_logo=True,
+        aspect=None,
+        backgroundcolor='cbs:highchartslichtgrijs',
+        logo_fillcolor='cbs:highchartslichtgrijs',
+        logo_edgecolor='cbs:logogrijs',
 ):
     """
-    Add a background to the axis label
+    Add a background to the axis labels.
 
     Parameters
     ----------
-    fig : `mpl.figure.Figure` object
+    fig : `matplotlib.figure.Figure` object
         The total canvas of the Figure
-    axes : `mpl.axes.Axes` object
+    axes : `matplotlib.axes.Axes` object
         The axes of the plot to add a box
-    alpha: float, optional
-        Transparency of the box. Default = 1 (not transparent)
-    margin: float, optional
-        The margin between the labels and the side of the gray box
-    loc: {"east", "south"}
-        Location of the background. Default = "east" (left to y-axis. Only "east" and "south" are
-        implemented
-    add_logo: bool, optional
-        If true, add the cbs logo. Default = True
-    radius_corner_in_mm: float, optional
-        Radius of the corner in mm. Default = 2
-    logo_margin_x_in_mm: float
-        Distance from bottom of logo in mm. Default = 2
-    logo_margin_y_in_mm=2,
-        Distance from left of logo in mm. Default = 2
-    backgroundcolor: str, optional
-        Color of the grey square. Default is "cbs:lichtgrijs"
-    logo_edgecolor: str, optional
-        Color of the logo edge. Default "cbs:logogrijs"
-    """
+    alpha : float, optional
+        The transparency of the background.
+        Default = 1
+    margin : float, optional
+        The margin between the axis labels and the border of the box in axes fraction coordinates.
+        Default = 0.05
+    x0 : float, optional
+        The x-coordinate of the bottom left corner of the gray square in axes fraction coordinates.
+        Default = None
+    y0 : float, optional
+        The y-coordinate of the bottom left corner of the gray square in axes fraction coordinates.
+        Default = None
+    loc : str, optional
+        The location of the gray box. Only "east" and "south" are implemented.
+        Default = "east"
+    radius_corner_in_mm : float, optional
+        The radius of the corner in mm. Default = 1
+    logo_margin_x_in_mm : float, optional
+        The margin between the left of the Figure and the logo in mm. Default = 1
+    logo_margin_y_in_mm : float, optional
+        The margin between the bottom of the Figure and the logo in mm. Default = 1
+    add_logo : bool, optional
+        Whether to add the CBS logo to the plot.
+        Default = True
+    aspect : float, optional
+        The aspect ratio of the plot. If None, the aspect ratio of the Figure is used.
+        Default = None
+    backgroundcolor : str, optional
+        The background color of the plot. Default = "cbs:highchartslichtgrijs"
+    logo_fillcolor : str, optional
+        The fill color of the logo. Default = "cbs:highchartslichtgrijs"
+    logo_edgecolor : str, optional
+        The edge color of the logo. Default = "cbs:logogrijs"
 
-    # the bounding box with respect to the axis in Figure coordinates
-    # (0 is bottom left canvas, 1 is top right)
+    Returns
+    -------
+    None
+
+    """
     bbox_axis_fig = axes.get_window_extent().transformed(fig.dpi_scale_trans.inverted())
 
     # the bounding box with respect to the axis coordinates
@@ -382,7 +453,7 @@ def add_axis_label_background(
         axes.transAxes.inverted()
     )
 
-    if loc == "east":
+    if loc == 'east':
         if x0 is None:
             x0 = bbox_axi.x0 - margin * bbox_axi.width
         x1 = 0
@@ -390,7 +461,7 @@ def add_axis_label_background(
         y0 = 0
         y1 = 1
 
-    elif loc == "south":
+    elif loc == 'south':
         x0 = 0
         x1 = 1
 
@@ -402,25 +473,25 @@ def add_axis_label_background(
             f"Location loc = {loc} is not recognised. Only east and south implemented"
         )
 
-    # width and height of the grey box area
+    # width and height of the gray box area
     width = x1 - x0
     height = y1 - y0
 
     _logger.debug(f"Adding rectangle with width {width} and height {height}")
 
-    # eerste vierkant zorgt voor rechte hoeken aan de rechter kant
-    if loc == "east":
+    # eerste vierkant zorgt voor rechte hoeken aan de rechterkant
+    if loc == 'east':
         rec_p = (x0 + width / 2, y0)
         rec_w = width / 2
         rec_h = height
-    elif loc == "south":
+    elif loc == 'south':
         rec_p = (x0, y0 + height / 2)
         rec_w = width
         rec_h = height / 2
     else:
         raise AssertionError(f"This should not happen")
 
-    p1 = mpatches.Rectangle(
+    p1 = m_patches.Rectangle(
         rec_p,
         width=rec_w,
         height=rec_h,
@@ -432,7 +503,7 @@ def add_axis_label_background(
     p1.set_transform(axes.transAxes)
     p1.set_clip_on(False)
 
-    # tweede vierkant zorgt voor ronde hoeken aan de linker kant
+    # tweede vierkant zorgt voor ronde hoeken aan de linkerkant
     radius_in_inch = radius_corner_in_mm / 25.4
     xshift = radius_in_inch / bbox_axis_fig.width
     yshift = radius_in_inch / bbox_axis_fig.height
@@ -441,7 +512,7 @@ def add_axis_label_background(
     if aspect is None:
         aspect = bbox_axis_fig.height / bbox_axis_fig.width
     _logger.debug(f"Using aspect ratio {aspect}")
-    p2 = mpatches.FancyBboxPatch(
+    p2 = m_patches.FancyBboxPatch(
         (x0 + xshift, y0 + yshift),
         width=width - 2 * xshift,
         height=height - 2 * yshift,
@@ -452,13 +523,10 @@ def add_axis_label_background(
         transform=fig.transFigure,
         zorder=0,
     )
-    p2.set_boxstyle("round", pad=pad)
+    p2.set_boxstyle('round', pad=pad)
     p2.set_transform(axes.transAxes)
     p2.set_clip_on(False)
 
-    # add_artist wordt niet door mpld3 herkend. add_patch wel. Wel chechekn of het ong wel goed gaat
-    # axes.add_artist(p1)
-    # axes.add_artist(p2)
     axes.add_patch(p1)
     axes.add_patch(p2)
 
@@ -481,12 +549,12 @@ def clean_up_artists(axis, artist_list):
     """
     try to remove the artists stored in the artist list belonging to the 'axis'.
     :param axis: clean artists belonging to these axis
-    :param artist_list: list of artist to remove
+    :param artist_list: list of artists to remove
     :return: nothing
     """
     for artist in artist_list:
         try:
-            # fist attempt: try to remove collection of contours for instance
+            # fist attempt: try to remove a collection of contours for instance
             while artist.collections:
                 for col in artist.collections:
                     artist.collections.remove(col)
@@ -505,3 +573,83 @@ def clean_up_artists(axis, artist_list):
             artist.remove()
         except (AttributeError, ValueError):
             pass
+
+
+def format_thousands_label(value: float, _: object) -> str:
+    """
+    Format a value with a thousand separator.
+
+    Parameters
+    ----------
+    value : float
+        The value to format.
+    _ : object
+        An unused parameter, only present to match the signature for a FormatStrFormatter.
+
+    Returns
+    -------
+    str
+        The formatted value with spaces as a thousand separator.
+
+    Examples
+    --------
+    >>> format_thousands_label(1234, None)
+    '1 234'
+    >>> format_thousands_label(1234567.89, None)
+    '1 234 567`
+    """
+    return '{:0,d}'.format(int(value)).replace(',', ' ')
+
+
+def swap_legend_boxes(
+        handles: List[m_patches.Patch], labels: List[str], n_cols: int
+) -> Tuple[List[m_patches.Patch], List[str]]:
+    """
+    Rearrange legend handles and labels to match the order of the first row.
+
+    In matplotlib, legend handles and labels are filled column-wise, while in highcharts, they are filled row-wise.
+
+    Parameters
+    ----------
+    handles : list of matplotlib.patches.Patch
+        The list of legend handles.
+    labels : list of str
+        The list of legend labels.
+    n_cols : int
+        The number of columns in the legend.
+
+    Returns
+    -------
+    reordered_handles : list of matplotlib.patches.Patch
+        The rearranged list of legend handles.
+    reordered_labels : list of str
+        The rearranged list of legend labels.
+    """
+    reordered_handles = handles[:]
+    reordered_labels = labels[:]
+
+    if len(reordered_labels) != len(reordered_handles):
+        raise ValueError('Number of handles and labels must be equal.')
+
+    rows_per_col: Dict[int, int] = {}
+
+    # Calculate number of rows per column
+    for idx, _ in enumerate(labels):
+        col = idx % n_cols
+        rows_per_col[col] = rows_per_col.get(col, 0) + 1
+
+    current_index = 0
+    current_row = 0
+    for idx, (handle, label) in enumerate(zip(handles, labels)):
+        reordered_handles[current_index] = handle
+        reordered_labels[current_index] = label
+
+        col = idx % n_cols
+        num_rows = rows_per_col[col]
+        current_index += num_rows
+
+        if col == n_cols - 1:
+            current_row += 1
+            current_index = current_row
+
+    return reordered_handles, reordered_labels
